#include "install.hpp"
#include <QDebug>
#include <QDir>
#include <QFile>
#include <QTextStream>
#include <QtDBus>
#include <QDBusInterface>
#include <QDBusReply>

#include <cstdlib>
#include <string>

const QString password = "qwer1234";

Install::~Install()
{

}

bool Install::runShellBlock(const QString &cmd, QString &ret)
{
    ret.clear();
    QProcess process;
    process.start(cmd);
    process.waitForFinished();
    if (process.exitCode() == 0)
    {
        ret = process.readAllStandardOutput();
        qDebug() << "True: " << ret;
        return true;
    }
    ret = process.readAllStandardError();
    qDebug() << "False: " << ret;
    return false;
}
bool Install::runShellBlock(const QString &cmd)
{
    std::string tmp = cmd.trimmed().toStdString();
    return system(tmp.c_str()) == 0;
}

void Install::runShellDetach(const QString &cmd)
{
    QProcess process;
    process.start(cmd);
}

void Install::sedSubstitute(const QString &file, const QString &searchPattern, const QString &replacePattern)
{
    QFile F(file);
    if (!F.open(QIODevice::ReadWrite | QIODevice::Text))
    {
        qDebug() << "Failed to open file for reading and writing.";
        return;
    }

    QTextStream in(&F);
    QString fileContent = in.readAll();
    fileContent.replace(QRegExp(searchPattern), replacePattern);

    F.resize(0);
    in << fileContent;
    F.close();
}


void Install::vpManager(const QString softwareNameValue, const QString softwareVersionNum, const QString softwareArch, const QString desktopNameList)
{
    QString pkgName = softwareNameValue, karePkgName = softwareNameValue,
            pkgVersion = softwareVersionNum + "-kare", pkgVersionDropKare = softwareNameValue,
            archValue = softwareArch, desktopValue = desktopNameList;
    QDir vpdir{vpWS};
    if (!vpdir.exists())
    {
        const QString debiandir = vpWS + "/vp/DEBIAN";
        vpdir.mkpath(debiandir);
    }

    QFile vpCF(vpControlFile);
    if (!vpCF.open(QIODevice::WriteOnly | QIODevice::Text))
    {
        qDebug() << "Cannot open file for writing:" << vpCF.errorString();
        return;
    }
    QTextStream out(&vpCF);
    out << "Package: \n";
    out << "Version: \n";
    out << "Architecture: \n";
    out << "Maintainer: zhanglin <zhanglin@kylinos.cn>\n";
    out << "Installed-Size: 160\n";
    out << "Depends: kare\n";
    out << "Section: misc\n";
    out << "Priority: optional\n";
    out << "Description: virtual package.\n";
    vpCF.close();

    QFile vpPRF(vpPostRmFile);
    if (!vpPRF.open(QIODevice::WriteOnly | QIODevice::Text))
    {
        qDebug() << "Cannot open file for writing:" << vpPRF.errorString();
        return;
    }
    QTextStream out2(&vpPRF);
    out2.setCodec("UTF-8");
    out2 << "#!/bin/bash \n"
         "pkgName=\n"
         "pkgVersion=\n"
         "desktopName=\n"
         "DesktopPath=/home/*/Desktop\n"
         "SYSLANG=$(export | grep -w LANG)\n"
         "if [[ \"${SYSLANG}\" =~ \"zh_CN\" ]]; then\n";
    out2 << QString::fromLocal8Bit("\tDesktopPath = /home/*/桌面\n");
    out2 << "elif [[ \"${SYSLANG}\" =~ \"en_US\" ]]; then\n"
         "\tDesktopPath=/home/*/Desktop\n"
         "fi\n"
         "for each in \"${desktopName}\"; do\n"
         "\tif ls $DesktopPath | grep -q $each; then\n"
         "\t\trm -f $DesktopPath/$each\n"
         "\tfi\n"
         "done\n"
         "if kare -l | grep -q \"$pkgName\"; then\n"
         "\tkare -P \"$pkgName\"\n"
         "fi\n";
    vpPRF.close();

    QString addx = "chmod a+x " + vpPostRmFile;
    QString addx_ret;
    if (!runShellBlock(addx, addx_ret))
    {
        qDebug() << "chmod a+x failed" << addx_ret;
        return;
    }

    if (!vpCF.open(QIODevice::ReadWrite | QIODevice::Text))
    {
        qDebug() << "Cannot open file for writing:" << vpCF.errorString();
        return;
    }
    QTextStream in(&vpCF);
    QString fileContent = in.readAll();
    fileContent.replace(QRegExp("Package:"), "Package: " + pkgName);
    fileContent.replace(QRegExp("Version:"), "Version: " + pkgVersion);
    fileContent.replace(QRegExp("Architecture:"), "Architecture: " + archValue);
    vpCF.resize(0);
    in << fileContent;
    vpCF.close();

    if (!vpPRF.open(QIODevice::ReadWrite | QIODevice::Text))
    {
        qDebug() << "Cannot open file for writing:" << vpCF.errorString();
        return;
    }
    QString fileContent2 = out2.readAll();
    fileContent2.replace(QRegExp("pkgName="),  "pkgName=" + karePkgName);
    fileContent2.replace(QRegExp("pkgVersion="), "pkgVersion=" + pkgVersion);
    fileContent2.replace(QRegExp("desktopName="), "desktopName=\"" + desktopValue + "\"");
    vpPRF.resize(0);
    out2 << fileContent2;
    vpPRF.close();
}

DockerInstall::DockerInstall()
{

}

DockerInstall::~DockerInstall()
{

}

int DockerInstall::container_preinit()
{
    qputenv("DBUS_SESSION_BUS_ADDRESS", "unix:path=/run/user/1000/bus");
    QList<QString> sc_list
    {
        "who",
        "users"
    };
    QDBusInterface iface = QDBusInterface
                           ("com.kylin.kare.Sessionhost",
                            "/com/kylin/kare/Sessionhost",
                            "com.kylin.kare.Sessionhost",
                            QDBusConnection::sessionBus()
                           );
    QString tmpfile = "/tmp/containerInit.tmp";
    QFile TFile(tmpfile);
    if (!TFile.open(QIODevice::ReadWrite | QIODevice::Text))
    {
        qDebug() << "tmp file failed";
        return -2;
    }
    QString init_res;
    for (const QString &each : sc_list)
    {
        QDBusReply<QString> reply = iface.call("call", each);
        if (!reply.isValid())
        {
            qDebug() << "invalid";
            return -2;
        }
        QByteArray data = reply.value().toLocal8Bit();
        TFile.write(data);

        QString homepath = QDir::homePath();
        QString bin_str = homepath + "/.config/compatible-layer/bin/";
        QDir homebin(bin_str);
        if (!homebin.exists())
        {
            homebin.mkpath(bin_str);
        }
        QString efile = bin_str + each + "-kare";
        QFile EFile(efile);
        if (!EFile.open(QIODevice::ReadWrite | QIODevice::Text))
        {
            qDebug() << "-kare file failed";
        }
        QTextStream efout(&EFile);
        efout << "#!/bin/bash\n"
              "echo \"" + reply.value() + "\"\n";
        EFile.close();
        runShellBlock
        (
            "echo " + password + " | sudo -S chmod a+x " + efile
        );
        runShellBlock
        (
            "which " + each,
            init_res
        );
    }
    TFile.close();
    return 0;
}

int DockerInstall::container_init(const QString &pkgname)
{
    QList<QString> sc_list
    {
        "who",
        "users"
    };
    for (const QString &each : sc_list)
    {
        QString homepath = QDir::homePath();
        QString bin_str = homepath + "/.config/compatible-layer/bin/";
        QString efile = bin_str + each + "-kare";
        QFile EFile(efile);
        if (!EFile.exists())
        {
            continue;
        }
        QString cs = QString("ln -sf %1 /usr/bin/%2").arg(efile).arg(each);
        runShellBlock(cs);
    }
    QString init_res;
    QProcess proc;
    proc.start("bash", QStringList() << "-c" << "echo qwer1234 | sudo  -S apt update");
    proc.waitForFinished();
    if (!proc.exitCode() == 0)
    {
        qDebug() << "Error: " << proc.readAllStandardError();
    }
    else
    {
        qDebug() << "Right: " << proc.readAllStandardOutput();
    }
    qDebug() << "------------------------------";
    return 0;
    if (!runShellBlock("pkexec apt update", init_res))
    {
        qDebug() << "apt update failed";
        //        return -1;
    }
    QString apt_cmd = "pkexec apt install " + pkgname;
    if (!runShellBlock(apt_cmd, init_res))
    {
        qDebug() << "kare-apt failed";
        //        return -1;
    }
    runShellBlock
    (
        "pkexec ln -sf /usr/bin/hostAgent /usr/bin/xdg-open"
    );
    runShellBlock
    (
        "ln -sf /usr/bin/hostAgent /usr/bin/gvfs-open"
    );
    runShellBlock
    (
        "ln -sf /usr/bin/hostAgent /usr/bin/gio"
    );
    runShellBlock
    (
        "ln -sf /usr/bin/python2 /usr/bin/python"
    );
    QList<QString> browser_list
    {
        "/usr/bin/firefox",
        "/usr/bin/firefox-esr",
        "/usr/bin/google-chrome-stable",
        "/usr/bin/chromium-browser",
        "/usr/bin/lbrowser",
        "/usr/bin/browser360-cn-stable",
        "/usr/bin/qaxbrowser-safe-stable"
    };
    for (const QString &each : browser_list)
    {
        QFile file(each);
        if (!file.exists())
        {
            runShellBlock
            (
                "ln -sf /usr/bin/hostAgent each"
            );
        }
    }
    QDir root_dir("/root");
    if (!root_dir.exists())
    {
        runShellBlock
        (
            "mkdir /root"
        );
    }
    return 0;
}

BwrapInstall::BwrapInstall()
{

}

BwrapInstall::~BwrapInstall()
{

}
