﻿#include "agent.hpp"
#include <QtDebug>
#include <QStringList>
#include <string>
#include "qlog.hpp"
#include <QFile>
#include <QDir>
#include <QTextStream>
#include <QTimer>
#include <QRegularExpression>
#include <QSettings>
#include <vector>
#include <cctype>

void osTree(QString name){
    QDir pathA("/sysroot");
    QDir pathB("/ostree");
    if (pathA.exists() && pathB.exists())
    {
        QProcess proc;
        QString bin = "mount -t overlay overlay -o lowerdir=/bin,upperdir=/var/kare/" + name + "/overlay/upper/bin,workdir=/var/kare/" + name + "/overlay/work/bin /var/kare/" + name + "/merged/bin";
        proc.start(bin);
        if (!proc.waitForStarted())
        {
            qDebug() << "ERROR - overlay 构建失败";
            return ;
        }
        if (!proc.waitForFinished(1000 * 30))
        {
            qDebug() << "ERROR - overlay 构建超时";
            return ;
        }
        if (proc.exitCode() != 0)
        {
            qDebug() << "ERROR - overlay 构建失败";
            return ;
        }

        QString sbin = "mount -t overlay  overlay -o lowerdir=/sbin,upperdir=/var/kare/" + name + "/overlay/upper/sbin,workdir=/var/kare/" + name + "/overlay/work/sbin /var/kare/" + name + "/merged/sbin";
        proc.start(sbin);
        if (!proc.waitForStarted())
        {
            qDebug() << "ERROR - overlay 构建失败";
            return ;
        }
        if (!proc.waitForFinished(1000 * 30))
        {
            qDebug() << "ERROR - overlay 构建超时";
            return ;
        }
        if (proc.exitCode() != 0)
        {
            qDebug() << "ERROR - overlay 构建失败";
            return ;
        }

        QString lib = "mount -t overlay  overlay -o lowerdir=/lib,upperdir=/var/kare/" + name + "/overlay/upper/lib,workdir=/var/kare/" + name + "/overlay/work/lib /var/kare/" + name + "/merged/lib";
        proc.start(lib);
        if (!proc.waitForStarted())
        {
            qDebug() << "ERROR - overlay 构建失败";
            return ;
        }
        if (!proc.waitForFinished(1000 * 30))
        {
            qDebug() << "ERROR - overlay 构建超时";
            return ;
        }
        if (proc.exitCode() != 0)
        {
            qDebug() << "ERROR - overlay 构建失败";
            return ;
        }

        QString lib64 = "mount -t overlay  overlay -o lowerdir=/lib64,upperdir=/var/kare/" + name + "/overlay/upper/lib64,workdir=/var/kare/" + name + "/overlay/work/lib64 /var/kare/" + name + "/merged/lib64";
        proc.start(lib64);
        if (!proc.waitForStarted())
        {
            qDebug() << "ERROR - overlay 构建失败";
            return ;
        }
        if (!proc.waitForFinished(1000 * 30))
        {
            qDebug() << "ERROR - overlay 构建超时";
            return ;
        }
        if (proc.exitCode() != 0)
        {
            qDebug() << "ERROR - overlay 构建失败";
            return ;
        }

        QString usr = "mount -t overlay  overlay -o lowerdir=/usr,upperdir=/var/kare/" + name + "/overlay/upper/usr,workdir=/var/kare/" + name + "/overlay/work/usr /var/kare/" + name + "/merged/usr";
        proc.start(usr);
        if (!proc.waitForStarted())
        {
            qDebug() << "ERROR - overlay 构建失败";
            return ;
        }
        if (!proc.waitForFinished(1000 * 30))
        {
            qDebug() << "ERROR - overlay 构建超时";
            return ;
        }
        if (proc.exitCode() != 0)
        {
            qDebug() << "ERROR - overlay 构建失败";
            return ;
        }

        QString  var = "mount -t overlay  overlay -o lowerdir=/var,upperdir=/var/kare/" + name + "/overlay/upper/var,workdir=/var/kare/" + name + "/overlay/work/var /var/kare/" + name + "/merged/var";
        proc.start(var);
        if (!proc.waitForStarted())
        {
            qDebug() << "ERROR - overlay 构建失败";
            return ;
        }
        if (!proc.waitForFinished(1000 * 30))
        {
            qDebug() << "ERROR - overlay 构建超时";
            return ;
        }
        if (proc.exitCode() != 0)
        {
            qDebug() << "ERROR - overlay 构建失败";
            return ;
        }
    }
    
}

Agent::Agent()
{
    qInstallMessageHandler(myMessageOutput);
    installProc = new QProcess;
}
Agent::~Agent()
{
    if (installProc) {
        delete installProc;
        installProc = nullptr;
    }
}

int Agent::dockerAgentTest()
{
    return 1024;
}

int Agent::dockerAgentCreate(const QString uid, const QString name, const QString env,
                             const QString sharedDir, const QString image)
{
    dockerLaunchCmd = "docker create -u ";
    dockerLaunchCmd = dockerLaunchCmd + uid + " -it --name " + name + " " + env + " " + sharedDir + " " + image;
    qDebug() << "DEBUG - dockerAgentCreateCmd: " << dockerLaunchCmd;
    QProcess proc;
    proc.start(dockerLaunchCmd);
    if (!proc.waitForStarted())
    {
        qDebug() << "ERROR - dockerAgentCreate 启动失败";
        return -1;
    }
    if (!proc.waitForFinished(1000 * 5))
    {
        qDebug() << "ERROR - dockerAgentCreate 执行超时";
        return -404;
    }
    if (proc.exitCode() != 0)
    {
        qDebug() << "ERROR - dockerAgentCreate 执行失败";
        return -1;
    }
    qDebug() << "DEBUG - dockerAgentCreate 执行成功";
    return 0;
}
int Agent::dockerAgentCp(const QString src, const QString des)
{
    dockerLaunchCmd = "docker cp ";
    dockerLaunchCmd = dockerLaunchCmd + src + " " + des;
    qDebug() << "DEBUG - dockerAgentCpCmd: " << dockerLaunchCmd;
    QProcess proc;
    proc.start(dockerLaunchCmd);
    if (!proc.waitForStarted())
    {
        qDebug() << "ERROR - dockerAgentCp 启动失败";
        return -1;
    }
    if (!proc.waitForFinished(1000 * 30))
    {
        qDebug() << "ERROR - dockerAgentCp 执行超时";
        return -404;
    }
    if (proc.exitCode() != 0)
    {
        qDebug() << "ERROR - dockerAgentCp 执行失败";
        return -1;
    }
    qDebug() << "DEBUG - dockerAgentCp 执行成功";
    return 0;
}
int Agent::dockerAgentInspect(const QString name)
{
    dockerLaunchCmd = "docker inspect ";
    dockerLaunchCmd = dockerLaunchCmd + name;
    qDebug() << "DEBUG - dockerAgentInspectCmd: " << dockerLaunchCmd;
    QProcess proc;
    proc.start(dockerLaunchCmd);
    if (!proc.waitForStarted())
    {
        qDebug() << "ERROR - dockerAgentInspect 启动失败";
        return -1;
    }
    if (!proc.waitForFinished(1000 * 5))
    {
        qDebug() << "ERROR - dockerAgentInspect 执行超时";
        return -404;
    }
    if (proc.exitCode() != 0)
    {
        qDebug() << "WARNING - dockerAgentInspect 执行成功，容器" << name << "不存在";
        return -2;
    }
    qDebug() << "DEBUG - dockerAgentInspect 执行成功";
    return 0;
}
int Agent::dockerAgentRm(const QString name)
{
    dockerLaunchCmd = "docker rm -f ";
    dockerLaunchCmd = dockerLaunchCmd + name;
    qDebug() << "DEBUG - dockerAgentRmCmd: " << dockerLaunchCmd;
    QProcess proc;
    proc.start(dockerLaunchCmd);
    if (!proc.waitForStarted())
    {
        qDebug() << "ERROR - dockerAgentRm 启动失败";
        return -1;
    }
    if (!proc.waitForFinished(1000 * 5))
    {
        qDebug() << "ERROR - dockerAgentRm 执行超时";
        return -404;
    }
    if (proc.exitCode() != 0)
    {
        qDebug() << "ERROR - dockerAgentRm 执行失败";
        return -1;
    }
    qDebug() << "DEBUG - dockerAgentRm 执行成功";
    return 0;
}
int Agent::dockerAgentStart(const QString name)
{
    dockerLaunchCmd = "docker start ";
    dockerLaunchCmd = dockerLaunchCmd + name;
    qDebug() << "DEBUG - dockerAgentStartCmd: " << dockerLaunchCmd;
    osTree(name);
    QProcess proc;
    proc.start(dockerLaunchCmd);
    if (!proc.waitForStarted())
    {
        qDebug() << "ERROR - dockerAgentStart 启动失败";
        return -1;
    }
    if (!proc.waitForFinished(1000 * 5))
    {
        qDebug() << "ERROR - dockerAgentStart 执行超时";
        return -404;
    }
    if (proc.exitCode() != 0)
    {
        qDebug() << "ERROR - dockerAgentStart 执行失败";
        return -1;
    }
    qDebug() << "DEBUG - dockerAgentStart 执行成功";
    return 0;
}
int Agent::dockerAgentExec(const QString name, const QString cmd)
{
    std::string name_ = name.toStdString();
    std::string cmd_ = cmd.toStdString();
    std::string dockerLaunchCmd_ = "docker exec " + name_ + " /bin/bash -c \"" + cmd_ + "\"";
    qDebug() << "DEBUG - dockerAgentExecCmd: " << QString(dockerLaunchCmd_.data());
    int ret = system(dockerLaunchCmd_.data());
    if (1 == ret)
    {
        qDebug() << "ERROR - " << QString("程序命令为空");
        return -1;
    }
    else if (-1 == ret)
    {
        qDebug() << "ERROR - " << QString("创建命令子进程失败");
        return -1;
    }
    else if (0x7f00 == ret)
    {
        qDebug() << "ERROR - " << QString("命令错误，无法执行");
        return -1;
    }
    else
    {
        if (WIFEXITED(ret))
        {
            if (WEXITSTATUS(ret) != 0)
            {
                qDebug() << "ERROR - " << QString("程序异常结束，返回值：%1").arg(WEXITSTATUS(ret));
                return -1;
            }
        }
        else if (WIFSIGNALED(ret))
        {
            qDebug() << "ERROR - " << QString("程序被信号杀死，信号值：%1").arg(WTERMSIG(ret));
            return -1;
        }
        else if (WSTOPSIG(ret))
        {
            qDebug() << "ERROR - " << QString("程序被信号暂停，信号值：%1").arg(WSTOPSIG(ret));
            return -1;
        }
    }
    qDebug() << "DEBUG - dockerAgentExec 执行成功";
    return 0;
}

bool startsWith(const std::string& str, const std::string& prefix) {
    return str.substr(0, prefix.length()) == prefix;
}

int Agent::dockerAgentExecTerminal(const QString name, const QString cmd)
{
    std::string name_ = name.toStdString();
    std::string cmd_ = cmd.toStdString();
    std::string dockerLaunchCmd_ = "docker exec -it " + name_ + " /bin/bash -c '" + cmd_ + "'";
    qDebug() << "DEBUG - dockerAgentExecTerminal: " << QString(dockerLaunchCmd_.data());
    p = new QProcess(this);
    QString program = "mate-terminal"; // 或者 "xterm"

    QProcess process;
    QStringList arg;
    arg << "-c" << "echo $(getent passwd $(who | awk \"{print $1}\"))";
    process.start("bash",arg);
    process.waitForFinished();
    int exitCode = process.exitCode();
    if (process.exitCode() != 0)
    {
        return -1;
    }
    QString output = process.readAllStandardOutput();
    QStringList fields = output.split(":");
    QString homeDirectory;
    if (fields.size() > 5) {
        homeDirectory = fields.at(5); // 获取第六个字段
        qDebug() << "Home Directory:" << homeDirectory;
    } else {
        return -1;
    }
    QString authDir = homeDirectory+"/.Xauthority";
    QProcessEnvironment env = QProcessEnvironment::systemEnvironment();
    env.insert("DISPLAY", ":0");
    env.insert("XAUTHORITY",authDir);
    p->setProcessEnvironment(env);
    QStringList arguments;
    arguments << "-e" << QString(dockerLaunchCmd_.data());
    p->start(program,arguments);
    connect(p, &QProcess::readyReadStandardError, this, [&](){
        QByteArray output = p->readAllStandardError();
        qDebug()<<"error:"<<QString(output);
    });
    connect(p, &QProcess::readyReadStandardOutput, this, [&](){
        QByteArray output = p->readAllStandardOutput();
        qDebug()<<"output"<<QString(output);
    });
    if (!p->waitForStarted()) {
        return -1;
    }
    qDebug() << "DEBUG - dockerAgentExecTerminal 执行成功";

    return 0;

}
int Agent::dockerAgentRun(const QString name, const QString cmd)
{
    std::string name_ = name.toStdString();
    std::string cmd_ = cmd.toStdString();
    std::string _cmd = cmd_;
    dockerName = name;

    QString configFilePath = "/opt/compatibility_layer/config/build-kare-env.conf";
    QSettings settings(configFilePath, QSettings::IniFormat);
    bool closeDockerAfterAppExited = settings.value("CLOSE_DOCKER_AFTER_APP_EXITED", false).toBool();
    if(closeDockerAfterAppExited){
        //启动docker events监听
        qDebug() << "DEBUG - 关闭应用后退出容器选项已开启";
        dockerLaunchCmd = "docker events --filter 'event=exec_start' --filter 'event=exec_die'";
        if(installProc->state() == QProcess::NotRunning){
            installProc->start("bash",QStringList() << "-c" << dockerLaunchCmd);
        }
        else{
            qDebug() << "DEBUG - docker events事件过滤正在运行中";
        }
    }else{
        stopDockerEventsProcess();
    }

    // 获取HOME变量的值
    QString command = "getent passwd $(who |awk '{print $1}')| awk -F: '{print $6}' | uniq";
    QProcess proc;
    proc.start("bash",QStringList() << "-c" << command);
    proc.waitForFinished();
    QByteArray output = proc.readAllStandardOutput();
    QString result = QString::fromUtf8(output).trimmed();
    std::string homeDir = result.toStdString();
    std::string logFilePath = homeDir + "/.log/compatibility_layer/kare_" + name_ + ".log";
    QFile file(QString::fromStdString(logFilePath));
    if(!file.exists()){
        if(file.open(QIODevice::ReadWrite)){
            qDebug() << "file " + QString::fromStdString(logFilePath) + " created successfully";
            QFile::Permissions permissions = QFile::ReadOwner | QFile::WriteOwner | QFile::ReadGroup
                    | QFile::WriteGroup | QFile::ReadOther | QFile::WriteOther;
            file.setPermissions(permissions);
            file.close();
        }
    }
    std::vector<std::string> wl{"DBUS_SESSION_BUS_ADDRESS",
    "echo qwer1234",
    "steam"
    };
    bool flag = false;
    for (std::string each : wl) {
        if (startsWith(cmd_, each)) {
            flag = true;
            break;
        }
    }
    if (!flag) {
        _cmd.clear();
        int length = cmd_.size();
        for (int i = 0; i < length; i++) {
            if (cmd_[i] == '=') {
                for (int j = i; j < length; j++) {
                    _cmd.push_back(cmd_[j]);
                    if (cmd_[j] == ' ') {
                        i = j + 1;
                        break;
                    }
                }
            }
            if ( i < length && cmd_[i] == ' ') {
                if ( i + 1 < length && (cmd_[i + 1] == '/' ||  cmd_[i + 1] == '~' ||  cmd_[i + 1] == '-' || isdigit(cmd_[i + 1]))) {
                     _cmd.push_back(' ');
                     continue;
                }
                if ( i - 3 >= 0 && cmd_.substr(i - 3, 3) == "env") {
                     _cmd.push_back(' ');
                     continue;
                }
                for (int j = i + 2; j < length; j++) {
                    if (cmd_[j] == '/') {
                        _cmd.push_back('\\');
                        break;
                    }
                }
            }
            _cmd.push_back(cmd_[i]);
        }
    }
    std::string dockerLaunchCmd_ = "docker exec -d " + name_ + " /bin/bash -c \"" + _cmd
            + " > " + logFilePath + " 2>&1" + "\"";
    qDebug() << "DEBUG - dockerAgentRunCmd: " << QString(dockerLaunchCmd_.data());
    FILE *pF = popen(dockerLaunchCmd_.data(), "r");
    if (pF == NULL)
    {
        qDebug() << "ERROR - dockerAgentRun 执行失败";
        return -1;
    }
    qDebug() << "DEBUG - dockerAgentRun 执行成功";

    //读取docker events输出内容，获取id、容器名称等
    connect(installProc, &QProcess::readyReadStandardOutput, this, [&](){
        QByteArray output = installProc->readAllStandardOutput();
        QString outputStr = QString::fromUtf8(output).trimmed();

        if(!outputStr.isEmpty()){
            QString matchField = "name=" + dockerName;
            QStringList lines = outputStr.split("\n", QString::SkipEmptyParts);
            for(const QString &line : lines){
                if(line.contains("exec_start") && line.contains(matchField) && !line.contains("dbus-daemon-proxy")){
                    QRegularExpression regex("execID=(\\w+)");
                    QRegularExpressionMatch match = regex.match(line);

                    if(match.hasMatch()){
                        QString execID = match.captured(1);
                        execIDList.append(execID);
                        qDebug() << "DEBUG - 匹配到docker events中" + dockerName + "的execID：" + execID;
                    }
                    else{
                        qDebug() << "DEBUG - 未匹配到docker events中" + dockerName + "的execID";
                    }
                }

                QString exitCodeField = "exitCode=0";
                if(line.contains("exec_die") && line.contains(exitCodeField)){
                    QRegularExpression execIDRegex("execID=(\\w+)");
                    QRegularExpression dockerNameRegex("name=([a-zA-Z0-9_\\.-]+)");
                    QRegularExpressionMatch execMatch = execIDRegex.match(line);
                    QRegularExpressionMatch dockerNameMatch = dockerNameRegex.match(line);
                    if(execMatch.hasMatch() && dockerNameMatch.hasMatch()){
                        QString execID = execMatch.captured(1);
                        QString dockerName = dockerNameMatch.captured(1);
                        if(execIDList.contains(execID)){
                            qDebug() << "搜索到匹配字段:" << dockerName << "的execID:" << execID << ", 执行关闭容器。";
                            if(isAllChildProcessDead(dockerName)){
                                QProcess closeProc;
                                QString closeCmd = "docker stop " + dockerName;
                                closeProc.start(closeCmd);
                                closeProc.waitForFinished();
                                if (closeProc.exitCode() != 0)
                                {
                                    qDebug() << "ERROR - 容器" << dockerName <<  "关闭失败 ";
                                    return;
                                }
                                qDebug() << "DEBUG - 容器" << dockerName << "关闭成功";
                            }
                            else{
                                qDebug() << "DEBUG - 容器" << dockerName << "存在未退出的进程，暂不关闭";
                            }
                        }
                        else{
                            qDebug() << "DEBUG - execID不存在,无法进行关闭容器操作";
                            return;
                        }
                    }
                }
            }
        }
    });
    // 程序不能阻塞，需要注释掉下面这行
    pclose(pF);
    return 0;
}
QString Agent::dockerAgentPs()
{
    dockerLaunchCmd = "docker ps -a";
    QProcess process;
    process.start(dockerLaunchCmd);
    process.waitForFinished();
    int exitCode = process.exitCode();
    if (process.exitCode() != 0)
    {
        qDebug() << "ERROR - dockerAgentPs 执行失败，exitCode: " << exitCode;
        return "";
    }
    QByteArray data = process.readAllStandardOutput();
    QString output = QString::fromUtf8(data);
    qDebug() << "DEBUG - dockerAgentPs 返回值为:" << output;
    qDebug() << "DEBUG - dockerAgentPs 执行成功";
    return output;
}
QString Agent::dockerAgentImages()
{
    dockerLaunchCmd = "docker images";
    QProcess process;
    process.start(dockerLaunchCmd);
    process.waitForFinished();
    int exitCode = process.exitCode();
    if (process.exitCode() != 0)
    {
        qDebug() << "ERROR - dockerAgentImages 执行失败，exitCode: " << exitCode;
        return "";
    }
    QByteArray data = process.readAllStandardOutput();
    QString output = QString::fromUtf8(data);
    qDebug() << "DEBUG - dockerAgentImages 返回值为:" << output;
    qDebug() << "DEBUG - dockerAgentImages 执行成功";
    return output;
}
bool Agent::updateFile(bool flag, const QString file)
{
    QFile qf(file);
    if (!qf.exists())
    {
        qDebug() << "ERROR - " << file << " Not Exists";
        return false;
    }
    QString fileTmp = file;

    fileTmp.remove("KARE");
    QString fileBAK = fileTmp + "_BAK";
    QString fileKAREBAK = fileTmp + "_KAREBAK";

    QFile qf2;
    if (!qf2.exists(fileBAK) || !qf2.exists(fileKAREBAK))
    {
        qDebug() << "ERROR - " << fileBAK << " or " << fileKAREBAK << " Not Exists";
        return false;
    }

    std::string file_ = file.toStdString();
    std::string fileBAK_ = fileBAK.toStdString();
    std::string fileKAREBAK_ = fileKAREBAK.toStdString();
    std::string updateCmd = "";
    if (flag)
    {
        // updateCmd = "cp -fp " + fileKAREBAK_ + " " + file_;
        updateCmd = "ln -sf " + fileKAREBAK_ + " " + file_;
    }
    else
    {
        // updateCmd = "cp -fp " + fileBAK_ + " " + file_;
        updateCmd = "ln -sf " + fileBAK_ + " " + file_;
    }
    qDebug() << "DEBUG - updateCmd: " << QString(updateCmd.data());
    int ret = system(updateCmd.data());
    if (1 == ret)
    {
        qDebug() << "ERROR - " << QString("程序命令为空");
        return false;
    }
    else if (-1 == ret)
    {
        qDebug() << "ERROR - " << QString("创建命令子进程失败");
        return false;
    }
    else if (0x7f00 == ret)
    {
        qDebug() << "ERROR - " << QString("命令错误，无法执行");
        return false;
    }
    else
    {
        if (WIFEXITED(ret))
        {
            if (WEXITSTATUS(ret) != 0)
            {
                qDebug() << "ERROR - " << QString("程序异常结束，返回值：%1").arg(WEXITSTATUS(ret));
                return false;
            }
        }
        else if (WIFSIGNALED(ret))
        {
            qDebug() << "ERROR - " << QString("程序被信号杀死，信号值：%1").arg(WTERMSIG(ret));
            return false;
        }
        else if (WSTOPSIG(ret))
        {
            qDebug() << "ERROR - " << QString("程序被信号暂停，信号值：%1").arg(WSTOPSIG(ret));
            return false;
        }
    }
    qDebug() << "DEBUG - updateFile 执行成功";
    return true;
}

void Agent::stopDockerEventsProcess()
{
    QProcess process;
    QString cmd = "ps ax | grep -v 'grep' | grep 'docker events'";
    process.start("bash",QStringList() << "-c" << cmd);
    process.waitForFinished();

    if(process.exitCode() == 0){
        QStringList dockerProc = QString(process.readAllStandardOutput()).split('\n',QString::SkipEmptyParts);
        if(!dockerProc.isEmpty()){
            foreach(const QString &proc, dockerProc){
                QStringList fields = proc.simplified().split(' ',QString::SkipEmptyParts);
                if(!fields.isEmpty()){
                    QString pid = fields.first();
                    QProcess killProcess;
                    killProcess.start("kill -9 " + pid);
                    killProcess.waitForFinished();
                }
            }
            qDebug() << "DEBUG - docker events已经停止";
        }
        else {
            qDebug() << "DEBUG - docker events进程列表为空";
        }

    }
    else{
        qDebug() << "DEBUG - docker events未启动";
    }
}

bool Agent::isAllChildProcessDead(QString name)
{
    QString filePath = "/tmp/kare-proc/" + name +"_pid";
    QFile file(filePath);
    if(!file.open(QIODevice::ReadOnly | QIODevice::Text)){
        qDebug() << "Faild to open file:" << filePath;
        return false;
    }

    QStringList procIds;
    while (!file.atEnd()) {
        QString line = file.readLine();
        procIds.append(line.trimmed());
    }

    file.close();

    if(procIds.isEmpty())
        return true;

    foreach (const QString&pid, procIds){
        QProcess process;
        process.start(QString("pgrep %1").arg(pid));
        process.waitForFinished();

        if(process.exitCode() == 0){
            return false;
        }
    }

    return true;
}
