﻿#include "packageDBusManager.h"
#include <QDebug>
#include <QDBusConnectionInterface>
#include <QDBusContext>
#include <QCoreApplication>
#include <QFile>
#include <QFileInfo>
#include <QDir>


static const QStringList programWhiteList={
    "/usr/share/kylin-system-updater/kylin-system-updater",
    "/usr/bin/d-feet",
    "/usr/bin/kylin-installer",
    "/usr/bin/kylin-uninstaller",
};

packageDBusManager::packageDBusManager()
{

}

packageDBusManager::~packageDBusManager()
{

}

bool packageDBusManager::install(QString env, QString name)
{
    if(!isAllowedCaller())
    {
       return -EPERM;
    }

    // 判断ok2环境是否存在
    bool hasOK2Environment = false;
    QProcess searchProc;
    QString searchEnvironment = "kare -E";
    QObject::connect(&searchProc, &QProcess::readyReadStandardOutput, [&]() {
        QByteArray output = searchProc.readAllStandardOutput();
        QString outputStr = QString::fromLocal8Bit(output);
            QStringList lines = outputStr.split(QRegExp("\n")); // 按行分割

            foreach (const QString &line, lines) {
                if (line.contains("openkylin2.0")) {
                    hasOK2Environment = true;
                }
            }
    });

    searchProc.start(searchEnvironment);
    if (!searchProc.waitForStarted())
    {
        qDebug() << "ERROR - openkylin2.0环境查询失败";
        return -1;
    }
    if (!searchProc.waitForFinished(1000 * 5))
    {
        qDebug() << "ERROR - openkylin2.0环境查询超时";
        return -404;
    }
    if (searchProc.exitCode() != 0)
    {
        qDebug() << "ERROR - openkylin2.0环境查询失败";
        return -1;
    }

    QString immutablePath = "/sysroot";
    QDir immutableDir(immutablePath);

    if (hasOK2Environment) {
        qDebug() << "openkylin2.0构建环境已存在";
    } else {
        if(immutableDir.exists()){
            qDebug() << "当前为不可变系统，openkylin2.0构建环境不存在，开始构建";
            QProcess buildProc;
            QString buildCommand = "kare -b openkylin2.0";
            buildProc.start(buildCommand);
            if (!buildProc.waitForStarted())
            {
                qDebug() << "ERROR - openkylin2.0环境构建失败";
                return -1;
            }
            if (!buildProc.waitForFinished(1000 * 10))
            {
                qDebug() << "ERROR -openkylin2.0环境构建超时";
                return -404;
            }
            if (buildProc.exitCode() != 0)
            {
                qDebug() << "ERROR - openkylin2.0环境构建失败";
                return -1;
            }
            qDebug() << "openkylin2.0环境构建成功";
        }
    }

    installTimer = new QTimer();
    installProc = new QProcess();
    packageName = name;

    QObject::connect(installProc, &QProcess::readyReadStandardOutput, this, &packageDBusManager::updateInstallProgress);
    QObject::connect(installProc,static_cast<void(QProcess::*)(int,QProcess::ExitStatus)>(&QProcess::finished),this,
                [=](int exitCode,QProcess::ExitStatus exitStatus){
        qDebug() << "exitCode:" << exitCode << "exitStatus:" << exitStatus;
        if((exitCode == 0) && (exitStatus == QProcess::NormalExit)){
            currentProgress = 100;
            // installTimer->stop();
            if(installTimer) {
                delete installTimer;
                installTimer = nullptr;
            }
            emitInstallStatusChanged(packageName, currentProgress, 0, "软件包安装成功");
            emitInstallFinishedSignal(packageName,0);
            resetMemberVariables();
            if(installProc) {
                delete installProc;
                installProc = nullptr;
            }
        }
        else{
            // installTimer->stop();
            if(installTimer) {
                delete installTimer;
                installTimer = nullptr;
            }
            if(currentProgress > 95){
                emitInstallStatusChanged(packageName, currentProgress, -1, "包配置管理失败");
                emitInstallFinishedSignal(packageName,-1);
            }
            else if ( currentProgress > 0 && currentProgress <= 10){
                emitInstallStatusChanged(packageName, currentProgress, -2, "存在依赖冲突或依赖缺失");
                emitInstallFinishedSignal(packageName,-1);
            }
            else if ( currentProgress > 10 && currentProgress <= 95){
                emitInstallStatusChanged(packageName, currentProgress, -3, "配置依赖包失败");
                emitInstallFinishedSignal(packageName,-1);
            }
            resetMemberVariables();
            if(installProc) {
                delete installProc;
                installProc = nullptr;
            }
        }
    });

    // 通过定时器发送信号
    connect(installTimer, &QTimer::timeout, this, [=](){
        if(currentProgress < 100)
            emitInstallStatusChanged(packageName, currentProgress, 1, "");
    });
    installTimer->start(1000);

    QString commandLine = "/usr/bin/kare -e " + env + " -i " + name;

    installProc->start(commandLine);
    if (!installProc->waitForStarted())
    {
        qDebug() << "ERROR - 软件包" << packageName << "安装进程执行失败";
        return -1;
    }
//    connect(installProc, QOverload<int, QProcess::ExitStatus>::of(&QProcess::finished), this, [&](){
//        bool flag = false;
//        int status = -1;
//        QFile file("/opt/compatibility_layer/config/install.list");
//        if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
//            qWarning() << "Failed to open file:" << file.errorString();
//            emitInstallFinishedSignal(packageName,status);
//        }
//        QTextStream in(&file);
//        while (!in.atEnd()) {
//            QString line = in.readLine();
//            QStringList parts = line.split('=');
//            if (!parts.isEmpty()) {
//                QString tmpName = parts[0];
//                if (packageName.contains(tmpName)) {
//                    // 找到匹配的字段
//                    flag = true;
//                    status = 0;
//                }
//            }
//        }
//        if (!flag)
//        {
//            emitInstallFinishedSignal(packageName,status);
//            qDebug() << "ERROR - 软件包" + packageName + "安装失败";
//            return;
//        }
//        emitInstallFinishedSignal(packageName,status);
//        qDebug() << "DEBUG - 软件包" + packageName + "安装成功";
//    });
    return true;
}

bool packageDBusManager::remove(QString name)
{
    if(!isAllowedCaller())
    {
        return -EPERM;
    }

    removeProc = new QProcess();

    QString commandLine = "/usr/bin/kare -P " + name;

    removeProc->start(commandLine);
    packageName = name;
//    connect(removeProc, QOverload<int, QProcess::ExitStatus>::of(&QProcess::finished), this, [&](){
    QObject::connect(removeProc,static_cast<void(QProcess::*)(int,QProcess::ExitStatus)>(&QProcess::finished),this,
                [=](int exitCode,QProcess::ExitStatus exitStatus){
        if((exitCode == 0) && (exitStatus == QProcess::NormalExit)){
            bool flag = true;
            int status = 0;
            QFile file("/opt/compatibility_layer/config/install.list");
            if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
                qWarning() << "Failed to open file:" << file.errorString();
                status = -1;
                emitRemoveFinishedSignal(packageName,status);
                emitRemoveStatusChanged(packageName,status,"资源配置缺失，卸载失败");
                if(removeProc) {
                    delete removeProc;
                    removeProc = nullptr;
                }
                return;
            }
            QTextStream in(&file);
            while (!in.atEnd()) {
                QString line = in.readLine();
                QStringList parts = line.split('=');
                if (!parts.isEmpty()) {
                    QString tmpName = parts[0];
                    if (packageName == tmpName) {
                        // 找到匹配的字段
                        flag = false;
                        status = -1;
                    }
                }
            }
            if (!flag)
            {
                emitRemoveFinishedSignal(packageName,status);
                emitRemoveStatusChanged(packageName,status,"资源配置缺失，卸载失败");
                qDebug() << "ERROR - 软件包" + packageName + "卸载失败";
                if(removeProc) {
                    delete removeProc;
                    removeProc = nullptr;
                }
                return;
            }
            emitRemoveFinishedSignal(packageName,status);
            emitRemoveStatusChanged(packageName,status,"");
            qDebug() << "DEBUG - 软件包" + packageName + "卸载成功";
            if(removeProc) {
                delete removeProc;
                removeProc = nullptr;
            }
        }
        else{
            emitRemoveFinishedSignal(packageName,-1);
            emitRemoveStatusChanged(packageName,-1,"资源配置缺失，卸载失败");
            qDebug() << "ERROR - 软件包" + packageName + "卸载失败";
            if(removeProc) {
                delete removeProc;
                removeProc = nullptr;
            }
            return;
        }
    });
    return true;
}

bool packageDBusManager::buildBaseEnv(QString baseEnv)
{
    if(!isAllowedCaller())
    {
        return -EPERM;
    }

    QString command = "kare -b " + baseEnv;

    timer = new QTimer();
    buildProc = new QProcess();
    baseEnvironment = baseEnv;
    if(baseEnv == QString("openkylin2.0")){
        QObject::connect(buildProc,static_cast<void(QProcess::*)(int,QProcess::ExitStatus)>(&QProcess::finished),this,
                 [=](int exitCode,QProcess::ExitStatus exitStatus){
            if((exitCode == 0) && (exitStatus == QProcess::NormalExit)){
                currentProgress = 100;
                // timer->stop();
                if(timer) {
                    delete timer;
                    timer = nullptr;
                }
                emitBuildStatusChangedSignal(baseEnvironment, currentProgress, 0, "");
                currentProgress = 0;
                if(buildProc) {
                    delete buildProc;
                    buildProc = nullptr;
                }
            }
            else{
                currentProgress = 80;
                // timer->stop();
                if(timer) {
                    delete timer;
                    timer = nullptr;
                }
                emitBuildStatusChangedSignal(baseEnvironment, currentProgress, -1, "构建失败，请确保网络稳定后，再次创建");
                currentProgress = 0;
                if(buildProc) {
                    delete buildProc;
                    buildProc = nullptr;
                }
            }
        });
    }else{
        QObject::connect(buildProc, &QProcess::readyReadStandardOutput, this, &packageDBusManager::updateProgress);
        QObject::connect(buildProc,static_cast<void(QProcess::*)(int,QProcess::ExitStatus)>(&QProcess::finished),this,
                 [=](int exitCode,QProcess::ExitStatus exitStatus){
            qDebug() << "exitCode:" << exitCode << "exitStatus:" << exitStatus;
            if((exitCode == 0) && (exitStatus == QProcess::NormalExit)){
                currentProgress = 100;
                // timer->stop();
                if(timer) {
                    delete timer;
                    timer = nullptr;
                }
                emitBuildStatusChangedSignal(baseEnvironment, currentProgress, 0, "");
                resetMemberVariables();
                if(buildProc) {
                    delete buildProc;
                    buildProc = nullptr;
                }
            }
            else{
                // timer->stop();
                if(timer) {
                    delete timer;
                    timer = nullptr;
                }
                if(currentProgress > 95)
                    emitBuildStatusChangedSignal(baseEnvironment, currentProgress, -1, "docker容器创建失败");
                else
                    emitBuildStatusChangedSignal(baseEnvironment, currentProgress, -1, "构建失败，请确保网络稳定后，再次创建");
                resetMemberVariables();
                if(buildProc) {
                    delete buildProc;
                    buildProc = nullptr;
                }
            }
        });
    }
    buildProc->start(command);
    if (!buildProc->waitForStarted())
    {
        qDebug() << "ERROR - 兼容环境环境构建失败";
        if(buildProc) {
            delete buildProc;
            buildProc = nullptr;
        }
        if(timer) {
            delete timer;
            timer = nullptr;
        }
        return false;
    }
    // 通过定时器发送信号
    connect(timer, &QTimer::timeout, this, [=](){
        if(currentProgress < 100)
            emitBuildStatusChangedSignal(baseEnvironment, currentProgress, 1, "");
    });
    timer->start(1000);
    return true;
}

void packageDBusManager::updateInstallProgress()
{
    QByteArray output = installProc->readAllStandardOutput();
    QString outputStr = QString::fromLocal8Bit(output);
    QStringList lines = outputStr.split(QRegExp("\n")); // 按行分割

    foreach (const QString &line, lines) {
        //判断系统中是否存在同名软件包
        if (line.contains("宿主系统上已安装同名软件") || line.contains("同名软件已存在于其他兼容环境")){
            emitInstallStatusChanged(packageName, 0, -4, line);
            if(installTimer) {
                delete installTimer;
                installTimer = nullptr;
            }
            resetMemberVariables();
            if(installProc) {
                delete installProc;
                installProc = nullptr;
            }
            QCoreApplication::quit();
        }
        // 第一阶段：获取源
        if (firstStage) {
            if (line.contains("DEBUG - 容器") || line.contains("命中") || line.contains("获取")) {
                totalLinesRead++;
                newProgress = totalLinesRead;
                newProgress = qMin(newProgress, 10); // 确保进度不超过10

                if (newProgress > currentProgress) {
                    currentProgress = newProgress;
                }
            } else if (line.contains("将会同时安装下列软件")){
                if(currentProgress < 10)
                    currentProgress = 10;
                firstStage = false;
                secondStage = true;
                firstStageLines = totalLinesRead;
            }
        }
        // 第二阶段：更新安装、升级依赖包
        else if (secondStage) {
            if (line.contains("获取")) {
                totalLinesRead++;
                newProgress = ((totalLinesRead - firstStageLines) / 5) + 10;
                newProgress = qMin(newProgress, 25);

                if (newProgress > currentProgress) {
                    currentProgress = newProgress;
                }
            } else if(line.contains("准备解压") || line.contains("正在解压")){
                if(currentProgress < 25)
                    currentProgress = 25;
                secondStage = false;
                thirdStage = true;
                theFirstTwoStageLines = totalLinesRead;
            }
        }
        // // 第三阶段：配置依赖包
        // else if (thirdStage) {
        //     if (line.contains("准备解压") || line.contains("正在解压") || line.contains("正在设置")) {
        //         totalLinesRead++;
        //         newProgress = ((totalLinesRead - theFirstTwoStageLines) / 5) + 25;
        //         newProgress = qMin(newProgress, 95);

        //         if (newProgress > currentProgress) {
        //             currentProgress = newProgress;
        //         }
        //     } else if(line.contains("正在设置 KARE 软件包管理")){
        //         if(currentProgress < 95)
        //             currentProgress = 95;
        //         thirdStage = false;
        //         fourthStage = true;
        //         theFirstThreeStageLines = totalLinesRead;
        //     }
        // }
        // 第四阶段：包管理
        if (thirdStage) {
           totalLinesRead++;
           newProgress = ((totalLinesRead - theFirstTwoStageLines) / 10) + 25;
           newProgress = qMin(newProgress, 99);

           if (newProgress > currentProgress) {
               currentProgress = newProgress;
           }
       }
    }
}

void packageDBusManager::updateProgress()
{
    QByteArray output = buildProc->readAllStandardOutput();
    QString outputStr = QString::fromLocal8Bit(output);
    QStringList lines = outputStr.split(QRegExp("\n")); // 按行分割

    foreach (const QString &line, lines) {
        // 第一阶段：检索验证软件包
        if (firstStage) {
            if (line.contains("Retrieving") || line.contains("Validating")) {
                totalLinesRead++;
                newProgress = totalLinesRead / 20; // 每25行增加1个进度
                newProgress = qMin(newProgress, 40); // 确保进度不超过40

                if (newProgress > currentProgress) {
                    currentProgress = newProgress;
                }
            } else if (line.contains("Extracting")){
                if(currentProgress < 40)
                    currentProgress = 40;
                firstStage = false;
                secondStage = true;
                firstStageLines = totalLinesRead;
            }
        }
        // 第二阶段：提取
        else if (secondStage) {
            if (line.contains("Extracting")) {
                totalLinesRead++;
                newProgress = ((totalLinesRead - firstStageLines) / 25) + 40;
                newProgress = qMin(newProgress, 45);
                if (newProgress > currentProgress) {
                    currentProgress = newProgress;
                }
            } else if(line.contains("Unpacking") || line.contains("Configuring")){
                if(currentProgress < 45)
                    currentProgress = 45;
                secondStage = false;
                thirdStage = true;
                theFirstTwoStageLines = totalLinesRead;
            }
        }
        // 第三阶段：解包配置
        else if (thirdStage) {
            if (line.contains("Unpacking") || line.contains("Configuring")) {
                totalLinesRead++;
                newProgress = ((totalLinesRead - theFirstTwoStageLines) / 30) + 45;
                newProgress = qMin(newProgress, 95);

                if (newProgress > currentProgress) {
                    currentProgress = newProgress;
                }
            } else if(line.contains("Base system installed successfully")){
                if(currentProgress < 95)
                    currentProgress = 95;
                thirdStage = false;
                fourthStage = true;
                theFirstThreeStageLines = totalLinesRead;
            }
        }
        // 第四阶段：拉取补充依赖
        if (fourthStage) {
           totalLinesRead++;
           newProgress = ((totalLinesRead - theFirstThreeStageLines) / 20) + 95;
           newProgress = qMin(newProgress, 99);

           if (newProgress > currentProgress) {
               currentProgress = newProgress;
           }
       }
    }
}

void packageDBusManager::resetMemberVariables()
{
    currentProgress = 0;
    totalLinesRead = 0;
    newProgress = 0;
    firstStageLines = 0;
    theFirstTwoStageLines = 0;
    theFirstThreeStageLines = 0;
    firstStage = true;
    secondStage = false;
    thirdStage = false;
    fourthStage = false;
}

bool packageDBusManager::checkEnviron(int pid) {
    QString filePath = QString("/proc/%1/environ").arg(pid);
    QFile file(filePath);
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
        qDebug() << "file.open failed" << filePath;
        return false;
    }
    QTextStream in(&file);
    QString data = in.readAll();
    file.close();
    data.replace('\0', '\n');
    QStringList environVars = data.split('\n');
    for (const QString &var : environVars){
        if (!var.isEmpty()) {
            int pos = var.indexOf('=');
            if (pos != -1) {
                QString key = var.left(pos);
                if (key == "LD_PRELOAD" || key == "LD_LIBRARY_PATH" || key == "LD_AUDIT"){
                    return false;
                }
            }
        }
    }
    return true;
}

bool packageDBusManager::isAllowedCaller(){
    return true;
    QString clientService = message().service();
    QDBusConnectionInterface *intf = connection().interface();
    unsigned int  pid = intf->servicePid(clientService).value();
    /*check caller environ*/
    if(!checkEnviron(pid)){
        sendErrorReply(QDBusError::ErrorType::Failed,QString("[%2] dbus method control,env forbidden").arg((pid)));
        return false;
    }
    /*check caller white list*/
    QFileInfo file(QString("/proc/%1/exe").arg(pid));
    QString execPath="";
    if (file.exists()) {
        QString linkFile = file.symLinkTarget();

        if (!linkFile.isNull()) {
            // 检查链接目标是否以 "/usr/bin/python" 开头（考虑版本号）
            if (linkFile.startsWith("/usr/bin/python")) {
                QFile cmdFile(QString("/proc/%1/cmdline").arg(pid));
                if (!cmdFile.open(QIODevice::ReadOnly | QIODevice::Text)) {
                    qDebug() << "Failed to open" << cmdFile;
                    return false;
                }

                QTextStream in(&cmdFile);
                QString cmdLine = in.readAll();

                // 移除末尾的 null 字符（如果有的话）
                while (cmdLine.endsWith(QChar('\0'))) {
                    cmdLine.chop(1);
                }

                // 将 null 字符替换为空格或其他分隔符，以便进一步处理
                cmdLine.replace(QChar('\0'), ' ');

                // 分割字符串并获取最后一个字段
               QStringList args = cmdLine.split(' ', QString::SkipEmptyParts);
               if (!args.isEmpty()) {
                   execPath = args.last(); // 返回最后一个字段
               }
            }
        }else{
            execPath= file.canonicalFilePath();
        }

        foreach (const QString& whitePath,programWhiteList){
            if (execPath == whitePath){
               return true;
            }
        }
    }
    sendErrorReply(QDBusError::ErrorType::Failed,QString("[%2] dbus method control,operation not permitted").arg((execPath)));
}

void packageDBusManager::emitInstallFinishedSignal(QString name, int status)
{
    QDBusMessage dbusMessage = QDBusMessage::createSignal("/org/kylin/kare", "org.kylin.kare", "installFinished");
    dbusMessage << QString(name) << status;
    QDBusConnection::systemBus().send(dbusMessage);
}

void packageDBusManager::emitRemoveFinishedSignal(QString name, int status)
{
    QDBusMessage dbusMessage = QDBusMessage::createSignal("/org/kylin/kare", "org.kylin.kare", "removeFinished");
    dbusMessage << QString(name) << status;
    QDBusConnection::systemBus().send(dbusMessage);
}

void packageDBusManager::emitBuildStatusChangedSignal(QString baseEnv, int progress, int status, QString errorInfo)
{
    QDBusMessage dbusMessage = QDBusMessage::createSignal("/org/kylin/kare", "org.kylin.kare", "buildStatusChanged");
    dbusMessage << QString(baseEnv) << progress << status << QString(errorInfo);
    QDBusConnection::systemBus().send(dbusMessage);
}

void packageDBusManager::emitInstallStatusChanged(QString name, int progress, int status, QString errorInfo)
{
    QDBusMessage dbusMessage = QDBusMessage::createSignal("/org/kylin/kare", "org.kylin.kare", "installStatusChanged");
    dbusMessage << QString(name) << progress << status << QString(errorInfo);
    QDBusConnection::systemBus().send(dbusMessage);
}

void packageDBusManager::emitRemoveStatusChanged(QString name, int status, QString errorInfo)
{
    QDBusMessage dbusMessage = QDBusMessage::createSignal("/org/kylin/kare", "org.kylin.kare", "removeStatusChanged");
    dbusMessage << QString(name) << status << QString(errorInfo);
    QDBusConnection::systemBus().send(dbusMessage);
}
