﻿#ifndef PACKAGEDBUSMANAGER_H
#define PACKAGEDBUSMANAGER_H

#include <QObject>
#include <QProcess>
#include <QDBusContext>
#include <QTimer>

class packageDBusManager : public QObject, protected QDBusContext
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface", "org.kylin.kare")
public:
    explicit packageDBusManager();
    ~packageDBusManager();

    bool checkEnviron(int pid);
    bool isAllowedCaller();
    void updateProgress();
    void resetMemberVariables();

    void emitInstallFinishedSignal(QString name, int status);
    void emitRemoveFinishedSignal(QString name, int status);
    void emitBuildStatusChangedSignal(QString baseEnv, int progress, int status, QString errorInfo);
    void emitInstallStatusChanged(QString name, int progress, int status, QString errorInfo);
    void emitRemoveStatusChanged(QString name, int status, QString errorInfo);

signals:
    void installFinished(QString name, int status);
    void removeFinished(QString name, int status);
    void buildStatusChanged(QString baseEnv, int progress, int status, QString errorInfo);
    void installStatusChanged(QString name, int progress, int status, QString errorInfo);
    void removeStatusChanged(QString name, int status, QString errorInfo);

public slots:
    bool install(QString env, QString name);
    bool remove(QString name);
    bool buildBaseEnv(QString baseEnv);

private:
    QProcess *installProc;
    QProcess *removeProc;
    QProcess *buildProc;
    QString packageName = "";
    QString baseEnvironment = "";
    QTimer *timer;
    QTimer *installTimer;
    int currentProgress = 0;
    int totalLinesRead = 0;    //跟踪已读取的行数
    int newProgress = 0;
    int firstStageLines = 0;
    int theFirstTwoStageLines = 0;
    int theFirstThreeStageLines = 0;
    bool firstStage = true;
    bool secondStage = false;
    bool thirdStage = false;
    bool fourthStage = false;
    void updateInstallProgress();
};

#endif // PACKAGEDBUSMANAGER_H
